#ifndef BUFFEREDITERATOR_H

#define BUFFEREDITERATOR_H

#include "basiclist.h"
#include "iterator.h"

template <class element>
class buffered_iterator : protected basic_list<element>
{
 private:
    iterator<element> internal_it;
    buffered_iterator<element> *m_pAncestor;
 private:
    void Update();
 protected:
    basic_list<element> const *internal_field;
    virtual void retrieve_elements()=NULL;
    inline void list_reset() { basic_list<element>::reset(); }
    inline void list_add(element e) { *this += e; }
    inline element *list_get_field() { return get_field(); }
    inline int list_cn() { return card(); }
    inline void list_touch() {	touch(); }
    inline int list_get_last_access() { return get_last_access(); } 
    inline void list_get_members(basic_list<element> const *L) { get_members(L); }
    void PushPosition();
    void PopPosition();
    inline iterator<element> &InternalIterator() { return internal_it; }
 public:
    buffered_iterator();
    ~buffered_iterator();
    void operator<<(basic_list<element> const *L);
    element operator*();
    element operator++(int);
    element operator--(int);
    int card();
    element operator[] (int nPos);
    element *get_field();
    void reset();
    void operator<<(buffered_iterator<element> *pIterator);
};

//---------------------------------------------------------------------------------------
// Implementation Area
//---------------------------------------------------------------------------------------

template <class element>
void buffered_iterator<element>::PushPosition()
{
    internal_it.PushPosition();
}

template <class element>
void buffered_iterator<element>::PopPosition()
{
    internal_it.PopPosition();
}

template <class element>
void buffered_iterator<element>::operator<<(buffered_iterator<element> *pIterator)
{
    *this << (basic_list<element> *)pIterator;

    m_pAncestor = pIterator;
}

template <class element>
buffered_iterator<element>::buffered_iterator()
{
    internal_it << this;
    internal_field = NULL;
    m_pAncestor = NULL;
}

template <class element>
buffered_iterator<element>::~buffered_iterator()
{

}

template <class element>
void buffered_iterator<element>::operator<<(basic_list<element> const *L)
{
    internal_field = L;
    internal_it.reset();
    retrieve_elements();
    m_pAncestor = NULL;
}  

template <class element>
void buffered_iterator<element>::Update()
{
    if (m_pAncestor != NULL)
    {
	m_pAncestor->Update();
    }

    if(internal_field&&!IsNewerThan(*internal_field))
    {
	retrieve_elements();
    }
}

template <class element>
element buffered_iterator<element>::operator*()
{
    Update();

    return *internal_it;
}

template <class element>
element buffered_iterator<element>::operator++(int)
{
    Update();

    return internal_it++;
}

template <class element>
element buffered_iterator<element>::operator--(int)
{
    Update();

    return internal_it--;
}
    
template <class element>
int buffered_iterator<element>::card()
{ 
    Update();

    return basic_list<element>::card();
}
   	
template <class element>
element buffered_iterator<element>::operator[] (int nPos)
{
    Update();

    if (nPos < card())
    {
	return get_field()[nPos];
    }
    else
    {
	return NULL;
    }
}
   	
template <class element>
element *buffered_iterator<element>::get_field()
{
    Update();

    return basic_list<element>::get_field();
}
    
template <class element>
void buffered_iterator<element>::reset()
{
    internal_it.reset();
}

#endif






