/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "cstrings.h"
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include "macrodefinitions.h"

CStringCore::CStringCore()
{
    m_nRefCount = 1;
    m_cString = NULL;
    m_nLength = 0;
    m_nInternalLength = 0;
}

CStringCore::~CStringCore()
{
    ASSERT(m_nRefCount == 0);
	
    if ( m_cString != NULL )
    {
	delete m_cString;
    }
}

CStrings::CStrings()
{
    // set the string value to an initial value

    CommonConstruct();
}

void CStrings::CommonConstruct()
{
    /*m_cString = NULL;
      m_nLength = 0;
      m_nInternalLength = 0;*/
  
    m_pCore = NULL;
}

char CStrings::operator[](int nIndex) const
{
    if (nIndex >= 0 && nIndex < Length())
    {
	char cPlatform;

	cPlatform = ((const char *)(*this))[nIndex];

	return cPlatform;
    }
    else
    {
	return 0;
    }
}

bool CStrings::operator==(const CStrings& strData) const
{
    return operator==((const char *)strData);
}

/*!
 *
 * \fn void CStrings::operator=(char *cData)
 *
 * \author STH
 * \date 25.3.00
 *
 * \brief sets the string to a specific value
 *
 */

void CStrings::operator=(const char *cData)
{
    if (cData != NULL)
    {
	/*if ( m_nLength < strlen(cData) ){
	  m_nLength = strlen(cData)+1;

	  if ( m_cString != NULL )
	  {
	  delete m_cString;
	  }
    
	  m_cString = new char[m_nLength];
	  }*/
  		
	int nLength = strlen(cData);
	FetchBuffer(nLength+1);

	ASSERT (m_pCore->String() != NULL);

	strncpy(m_pCore->String(), cData, nLength);
	(m_pCore->String())[nLength] = '\0';

	SetLength(nLength);
    }
    else
    {
	UnRef();
  		
	SetLength(0);
	//delete m_cString;
	//m_cString = NULL;
	if (m_pCore != NULL && m_pCore->String() != NULL)
	{
	    *(m_pCore->String()) = '\0';
	}
    }
}

CStrings::CStrings(const char *cData1, const char *cData2)
{
    CommonConstruct();
	
    Concatenate(cData1, cData2);
}

void CStrings::operator+=(char cData)
{
    char cBuffer[2];
    cBuffer[0] = cData;
    cBuffer[1] = '\0';

    *this += cBuffer;
}

void CStrings::operator=(char cData)
{
    char cBuffer[2];
    cBuffer[0] = cData;
    cBuffer[1] = '\0';

    *this = cBuffer;
}

void CStrings::Concatenate(const char *cData1, const char *cData2)
{
    int nLength = strlen(cData1);
    int nTotalLength = strlen(cData2)+nLength;
	
    FetchBuffer(nTotalLength);
	
    strcpy(m_pCore->String(), cData1);
    strcpy(m_pCore->String()+nLength,cData2);
	
    SetLength(nTotalLength);
}

CStrings::operator PCCHAR() const
{
    if ( m_pCore != NULL && m_pCore->String() != NULL )
    {
	return m_pCore->String();
    }
    else
    {
	return "";
    }
}

void CStrings::UnRef(bool bCopy)
{
    if ( m_pCore == NULL )
    {
	m_pCore = new CStringCore;
    }
    else if ( m_pCore->RefCount() > 1 )
    {
	CStringCore *pOldCore = m_pCore;
		
	m_pCore->DecreaseRefCount();
	m_pCore = new CStringCore;
		
	if ( bCopy )
	{
	    m_pCore->AllocMemory(pOldCore->InternalLength());
			
	    ASSERT(m_pCore->InternalLength() > strlen(m_pCore->String()));

	    strcpy(m_pCore->String(), pOldCore->String());	
	    m_pCore->SetLength(pOldCore->Length());
			
	    ASSERT(m_pCore->Length() == strlen(m_pCore->String()));		
	}
    }
}

void CStrings::LeftTrim(char c)
{
    ASSERT(m_pCore->InternalLength() >= strlen(m_pCore->String()));
    UnRef(true);

    char *pSource, *pDest;
	
    if (m_pCore != NULL && m_pCore->String() != NULL && strlen(m_pCore->String()) > 0 )
    {
	pDest = m_pCore->String();
	pSource = m_pCore->String();
	
	while(*pSource != '\0' && *pSource == c )
	{
	    pSource++;
	}
	
	if ( *pSource == '\0' )
	{
	    *pDest = '\0';
	}
	else
	{
	    do
	    {
		*pDest++ = *pSource++;
	    }while(*pSource != '\0');
		
	    *pDest = '\0';
	}
	
	SetLength(strlen(m_pCore->String()));
    }
}

void CStrings::RightTrim(char c)
{
    UnRef(true);
	
    if (m_pCore != NULL)
    {
	if ( m_pCore->String() != NULL && strlen(m_pCore->String()) > 0 )
	{
	    char *pZeichen = m_pCore->String() + strlen(m_pCore->String())-1;
	    do
	    {
		if ( *pZeichen != c )
		{
		    break;
		}
		else
		{
		    *pZeichen-- = '\0';
		}
	    }while(pZeichen > m_pCore->String());
	}
	
	SetLength(strlen(m_pCore->String()));
    }
}

void CStrings::operator=(const CStrings &string)
{
    // *this = (const char *)string;

    if ( m_pCore != NULL )
    {
	m_pCore->DecreaseRefCount();
		
	if ( m_pCore->RefCount() == 0 )
	{
	    delete m_pCore;
	}
    }
	
    m_pCore = string.m_pCore;
	
    if ( m_pCore != NULL )
    {
	m_pCore->IncreaseRefCount();
    }
}

void CStrings::Empty()
{
	*this = (char *)NULL;	
}

void CStrings::operator+=(const char *cData)
{
    FetchBuffer(Length()+strlen(cData));
	
    strcat(m_pCore->String(),cData);
    SetLength(strlen(m_pCore->String()));
}

void CStringCore::AllocMemory(int nLength)
{
    nLength++;
	
    if (InternalLength() < nLength)
    {
	// Add about 12.5 % characters
		
	int nPlus = nLength;
	nPlus = nPlus >> 3; // divide by 8		
	nLength += nPlus;
		
	bool bInit = false;
		
	bInit = (m_cString == NULL);
		
	m_cString = (char *)realloc(m_cString,nLength);
	
	if (bInit)
	{
	    *m_cString = '\0';
	}
		
	m_nInternalLength = nLength;
    }
}

void CStrings::FetchBuffer(int nLength)
{
    UnRef(true);
		
    m_pCore->AllocMemory(nLength);
}

void CStrings::SetLength(int nLength)
{
    ASSERT(m_pCore->InternalLength() >= nLength);
    ASSERT(m_pCore->RefCount() == 1);
	
    m_pCore->SetLength(nLength);
}

CStrings::~CStrings()
{
    /*if ( m_cString != NULL )
      {
      delete m_cString;
      }*/
	
    if ( m_pCore != NULL )
    {
	m_pCore->DecreaseRefCount();
		
	if ( m_pCore->RefCount() == 0 )
	{
	    delete m_pCore;
	    m_pCore = NULL;
	}
    }
}

CStrings::CStrings(const CStrings &S)
{
    CommonConstruct();
		
    if ( S.m_pCore )
    {
	*this = S;
    }
}

void CStrings::Format(const char *cFormat, ...)
{
    char *cBuffer;
    int nSize = 1024;
	
    cBuffer = new char[nSize];
    va_list argptr;
    va_start(argptr,cFormat);

#ifndef WIN32
    int nNewsize = vsnprintf(cBuffer,nSize,cFormat,argptr);
#else
    int nNewsize = _vsnprintf(cBuffer,nSize,cFormat,argptr);
#endif

    if ( nNewsize > nSize )
    {
	nSize = nNewsize+10;
	cBuffer = (char *)realloc(cBuffer,nSize);
	ASSERT(cBuffer != NULL);
#ifndef WIN32
	nNewsize = vsnprintf(cBuffer,nSize,cFormat,argptr);
#else
	nNewsize = _vsnprintf(cBuffer,nSize,cFormat,argptr);
#endif
    }
	
    ASSERT(nSize >= nNewsize);

    *this = cBuffer;
    delete cBuffer;
    va_end(argptr);
}

CCatString CStrings::operator+(const char *cData) const
{
	CCatString NeuString;
	//return CCatString((const char *)*this,cData);

    NeuString.Concatenate((const char *)*this, cData);

	return NeuString;
}

CCatString &CCatString::operator+(const char *cData)
{
    *this += cData;
	
    return *this;
}

void CCatString::operator=(const char *cData)
{
    CStrings::operator=(cData);
}

void CCatString::operator+=(const char *cData)
{
    CStrings::operator+=(cData);
}

/*CCatString::CCatString(const char *cData1, const char *cData2)
	: CStrings(cData1, cData2)
{

}*/

CCatString::CCatString()
{

}

CCatString operator+(const char *cData1, const CStrings &strData2)
{
    CCatString NeuString;
    //return CCatString(cData1,(const char *)strData2);

    NeuString.Concatenate(cData1, (const char *)strData2);

    return NeuString;
}

CCatString &CCatString::operator+(const CStrings &strData)
{
    return operator+((const char *)strData);
}

CCatString CStrings::operator+(const CStrings &strData) const
{
    return operator+((const char *)strData);
}

bool CStrings::operator==(const char *strData) const
{
    if (m_pCore != NULL)
    {
	const char *strString = m_pCore->String();

	if (strString != NULL)
	{
	    int nResult = strcmp(m_pCore->String(),strData);

	    return (nResult == 0);
	}
	else
	{
	    return ((strData == NULL) || (strData[0] == '\0'));
	}
    }
    else
    {
	bool bGleich = ((strData == NULL) || (strData[0] == '\0'));
	
	return bGleich;	
    }
}

bool operator==(const char *cData1, const CStrings &strData2)
{
    return strData2 == cData1;
}

CStrings::CStrings(const char *cString)
{
    CommonConstruct();
	
    *this = cString;
}

CStrings::CStrings(const char *cString, int nLength)
{
    CommonConstruct();	
	
    StringCopy(cString, nLength);
}

void CStrings::StringCopy(const char *cString, int nLength)
{
    FetchBuffer(nLength+1);
	
    if (strlen(cString) < (unsigned int)nLength)
    {
	nLength = strlen(cString);
    }

    if (cString != NULL)
    {
	strncpy(m_pCore->String(), cString, nLength);
	(m_pCore->String())[nLength] = '\0';

	SetLength(strlen(m_pCore->String()));
    }
    else
    {
	*this = "";
    }
}

CStrings CStrings::Left(int nPos) const
{
    CStrings NeuString;

    ASSERT(nPos >= 0 );
	
    if (m_pCore != NULL)
    {
	//return CString(m_pCore->String(),nPos);
	NeuString.StringCopy(m_pCore->String(), nPos);
    }

    return NeuString;
}

CStrings CStrings::Right(int nPos) const
{
    CStrings NeuString;

    ASSERT(nPos >= 0 );
    if (m_pCore != NULL)
    {
	char *cPos = m_pCore->String()+Length()-nPos;
	
	if ( cPos >= m_pCore->String() )
	{
	    //return CString(cPos);
		
	    NeuString = cPos;
	}
	else
	{
	    //return CString(m_pCore->String());
		
	    NeuString = *this;
	}
    }

    return NeuString;
}

CStrings CStrings::Mid(int nPosStart, int nPosEnd) const
{
    CStrings NeuString;

    ASSERT(nPosStart >=0 );
    //ASSERT(nPosStart <= Length());
    ASSERT(nPosEnd >= nPosStart || nPosEnd == -1);
	
    if (nPosStart >= Length())
    {
	NeuString = "";
    }
    else
    {
	if ( nPosEnd >= 0 )
	{
	    //return CString(m_pCore->String()+nPosStart,nPosEnd-nPosStart);
	    //NeuString.Copy(m_pCore->String()+nPosStart, nPosEnd-nPosStart);
	    NeuString.StringCopy(m_pCore->String()+nPosStart, nPosEnd-nPosStart+1);
	}
	else
	{
	    //return CString(m_pCore->String()+nPosStart);
		
	    NeuString = (m_pCore->String()+nPosStart);
	}
    }
    return NeuString;
}

int CStrings::FindReverse(char cData)
{
    if (m_pCore != NULL)
    {
	char *cPointer = m_pCore->String()+Length()-1;
		
	for (int nIndex = Length()-1; nIndex >= 0; nIndex--)
	{
	    if (*cPointer == cData)
	    {
		return nIndex;
	    }
	    else
	    {
		cPointer--;
	    } 
	}
    }
	
    return -1;
}

int CStrings::Find(char cData)
{
    if (m_pCore != NULL)
    {
	char *cPointer = m_pCore->String();
	
	int nLength = Length();
		
	for (int nIndex = 0; nIndex < nLength; nIndex++)
	{
	    if (*cPointer == cData)
	    {
		return nIndex;
	    }
	    else
	    {
		cPointer++;
	    }
	}
    }
	
    return -1;
}

/*CCatString operator+(const char *cData1, CStrings &strData2) return strResult
{
    strResult = cData1;
    strResult += strData2;
}*/

void CStrings::Replace(const char *strOldString, const char *strNewString)
{
    UnRef(true);

    char *strPosition = strstr((const char *)*this, strOldString);
    int nLength = strlen(strOldString);
    int nNewLength = strlen(strNewString);

    while (strPosition != NULL)
    {
	int nLeft = strPosition - (const char *)*this;

	*this = Left(nLeft)+strNewString+(strPosition+nLength);

// 	strPosition = strstr(((const char *)*this)+nLeft+nLength, strOldString);
	strPosition = strstr(((const char *)*this)+nLeft+nNewLength, strOldString);
    }
}

void CStrings::Set(const char *pBuffer, int nLength)
{
    char buffer[nLength+1];
    buffer[nLength] = 0;

    strncpy(buffer, pBuffer, nLength);

    *this = buffer;
}
