/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef STRING_H

#define STRING_H

#include <stdlib.h>
#include <string.h>
#include "identificator.h"

typedef const char * PCCHAR;

class CStringCore
{
    friend class CStrings;
 private:
    unsigned int m_nRefCount;
    char *m_cString;
    int m_nLength;
    int m_nInternalLength;
 private:
    inline int InternalLength() { return m_nInternalLength; }
 public:		
    inline char *String() { return m_cString; }
    inline int RefCount() { return m_nRefCount; }
    inline void DecreaseRefCount() { m_nRefCount--; }
    inline void IncreaseRefCount() { m_nRefCount++; }
    void AllocMemory(int nLength);
    inline void SetLength(int nLength) { m_nLength = nLength; }	
    inline int Length() const { return m_nLength; }

    CStringCore();
    ~CStringCore();
};

class CCatString;

class CStrings
{
    friend CCatString operator+(const char *cData1, const CStrings &strData2);
 private:
    /*char *m_cString;
      int m_nLength;
      int m_nInternalLength;*/
    CStringCore *m_pCore;
 private:
    void SetLength(int nLength);
    void FetchBuffer(int nLength);
    void CommonConstruct();
    void UnRef(bool bCopy = false);
    void StringCopy(const char *cString, int nLength);
 protected:
    CStrings(const char *cData1, const char *cData2);
    void Concatenate(const char *cData1, const char *cData2);
 public:
    CStrings();
    CStrings(const char *cString);
    CStrings(const CStrings & S);
    CStrings(const char *cString, int nLength);
		
    virtual ~CStrings();
  		
    inline int Length() const { return m_pCore != NULL ? m_pCore->Length() : 0;}
    void operator=(const char *cData);
    operator PCCHAR() const;
    void LeftTrim(char c=' ');
    void RightTrim(char c=' ');
  		
    inline void Trim(char c=' ') { LeftTrim(c), RightTrim(c); }
  		
    void operator=(const CStrings &string);
    void Empty();
    void operator += (const char *cData);
    void operator += (char cData);
    void operator = (char cData);
    CCatString operator + (const char *cData) const;
    CCatString operator + (const CStrings &strData) const;

    void Format(const char *cFormat, ...);
    bool operator==(const char *strData) const;
    bool operator==(const CStrings& strData) const;
    inline bool operator>(const CStrings& strData) const { return (strcmp((const char *)(*this), (const char *)strData) > 0); }
    inline bool operator<(const CStrings& strData) const { return (strcmp((const char *)(*this), (const char *)strData) < 0); }
    inline bool operator!=(const char *strData) const { return !operator==(strData); }
    CStrings Left(int nPos) const;
    CStrings Right(int nPos) const;
    CStrings Mid(int nPosStart, int nPosEnd=-1) const;
    int Find(char cData);
    int FindReverse(char cData);
    char operator[](int nIndex) const;

    void Replace(const char *strOldString, const char *strNewString);
    void Set(const char *pBuffer, int nLength);
};

class CCatString : public CStrings
{
 public:
    //CCatString(const char *cData1, const char *cData2);
    CCatString();
		
    CCatString &operator + (const char *cData);
    CCatString &operator + (const CStrings &strData);
    void operator=(const char *cData);
    void operator += (const char *cData);
};

//CCatString operator+(const char *cData1, const CStrings &strData2);

bool operator==(const char *cData1, const CStrings &strData2);

#endif






