/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "date.h"
#include "cstrings.h"
#ifdef _UNIX
#include <sys/time.h>
#endif
#include "macrodefinitions.h"
#include "globalstring.h"

const int CDate::m_pDays[] = {31,28,31,30,31,30,31,31,30,31,30,31};

void CDate::SetDate(int nDay, int nMonth, int nYear)
{
    if ( nMonth > 0 && nMonth < 13 )
    {
	switch(nMonth)
	{
	    case 4:
	    case 6:
	    case 9:
	    case 11:
      		if ( nDay > 0 && nDay < 31 )
      		{
		    if ( nYear >= 1970 )
		    {
			m_nDay = nDay-1;
			m_nMonth = nMonth-1;
			m_nYear = nYear-1970;
			SetValid();
		    }
		    else
		    {
			SetValid(false);
		    }
      		}
      		else
      		{
		    SetValid(false);
      		}
      		break;
	    case 2:
      		if ( Schaltjahr(nYear) )
      		{
		    if ( nDay > 0 && nDay < 30 )
		    {
			if ( nYear >= 1970 )
			{
			    m_nDay = nDay-1;
			    m_nMonth = nMonth-1;
			    m_nYear = nYear-1970;
			    SetValid();
			}
			else
			{
			    SetValid(false);
			}
		    }
		    else
		    {
			SetValid(false);
		    }
      		}
      		else
      		{
		    if ( nDay > 0 && nDay < 29 )
		    {
			if ( nYear >= 1970 )
			{
			    m_nDay = nDay-1;
			    m_nMonth = nMonth-1;
			    m_nYear = nYear-1970;
			    SetValid();
			}
			else
			{
			    SetValid(false);
			}	  
		    }
		    else
		    {
			SetValid(false);
		    }
      		}
      		break;
	    default:
      		if ( nDay > 0 && nDay < 32 )
      		{
		    if ( nYear >= 1970 )
		    {
			m_nDay = nDay-1;
			m_nMonth = nMonth-1;
			m_nYear = nYear-1970;
			SetValid();
		    }
		    else
		    {
			SetValid(false);
		    }
      		}
      		else
      		{
		    SetValid(false);
      		}      
      		break;
    	}
    }
    else
    {
    	SetValid(false);
    }
}

CDate::CDate(const char *strDate)
{
    SetDate(1,1,1970);
    SetValid(true);
    *this = strDate;
}

CDate::CDate()
{
    SetDate(1,1,1970);
    SetValid(true);
    //m_cBuffer = NULL;
}

void CDate::SetValid(bool bValid)
{
    if (bValid)
    {
    	m_nValid |= 0x1;
    }
    else
    {
    	m_nValid &= 0x0;
    }
}

bool CDate::Schaltjahr(int nYear) const
{
    if (!(nYear&0x1))
    {
	if ((nYear%1000) == 0)
	{
	    if ((nYear%400) == 0)
	    {
		return false;
	    }
	    else
	    {
		return true;
	    }
	}
	else
	{
	    if ((nYear%4) == 0)
	    {
		return true;
	    }
	    else
	    {
		return false;
	    }
	}
    }

    return false;
}

/*CDate::operator PCHAR()
{
    if ( m_cBuffer == NULL )
    {
	m_cBuffer = new char[13];
    }

    snprintf(m_cBuffer, 12, "%02d.%02d.%04d\0",Day(),Month(),Year());

    return m_cBuffer;
}*/

CDate::operator CStrings() const
{
    CStrings string;

    string.Format("%02d.%02d.%04d\0",Day(),Month(),Year());

    return string;
}

CDate::~CDate()
{
    /*if ( m_cBuffer )
      {
      delete m_cBuffer;
      }*/
}

void CDate::operator=(const char *cDatum)
{
    if (strlen(cDatum) >= 5)
    {
	SetValid(true);
	
	/*if ( strlen(cDatum) != 10 )
	  {
	  SetValid(false);
	  }
	  else
	  {
	  if ( cDatum[2] != '.' )
	  {
	  SetValid(false);
	  }
	  else
	  {
	  if ( cDatum[5] != '.' )
	  {
	  SetValid(false);
	  }
	  else
	  {
	  int nDay, nMonth, nYear;
				
	  nDay = (cDatum[0]-'0')*10+(cDatum[1]-'0');
	  nMonth = (cDatum[3]-'0')*10+(cDatum[4]-'0');
	  nYear = (cDatum[6]-'0')*1000+(cDatum[7]-'0')*100+(cDatum[8]-'0')*10+(cDatum[9]-'0');

	  SetDate(nDay, nMonth, nYear);
	  }
	  }
	  }*/

	CStrings strDate = cDatum;

	int nPoint = strDate.Find('.');
	if (nPoint > 0)
	{
	    CStrings strDay = strDate.Left(nPoint);
	    strDate = strDate.Mid(nPoint+1);
	    nPoint = strDate.Find('.');
	    if (nPoint > 0)
	    {
		CStrings strMonth = strDate.Left(nPoint);
		CStrings strYear = strDate.Mid(nPoint+1);

		int nDay = atoi((const char *)strDay);
		int nMonth = atoi((const char *)strMonth);
		int nYear = atoi((const char *)strYear);
	    
		if (nYear < 100 && nYear >= 0)
		{
		    if (nYear < 70)
		    {
			nYear += 2000;
		    }
		    else
		    {
			nYear += 1900;
		    }
		}

		SetDate (nDay, nMonth, nYear);
	    }
	    else
	    {
		SetValid(false);
	    }
	}
	else
	{
	    SetValid(false);
	}
    }
    else
    {
	SetValid(false);
    }
}

void CDate::operator=(const CDate &date)
{
    m_nDay = date.m_nDay;
    m_nMonth = date.m_nMonth;
    m_nYear = date.m_nYear;
    m_nValid = date.m_nValid;
}

CDate::CDate(const CDate &date)
{
    m_nDay = date.m_nDay;
    m_nMonth = date.m_nMonth;
    m_nYear = date.m_nYear;
    m_nValid = date.m_nValid;
    //m_cBuffer = NULL;
}

bool CDate::operator<(const CDate &date) const
{
    if (!Valid() && date.Valid())
    {
	return true;
    }
    else if (!date.Valid())
    {
	return false;
    }
    else
    {
	if (Year() < date.Year())
	{
	    return true;
	}
	else if (Year() == date.Year())
	{
	    if (Month() < date.Month())
	    {
		return true;
	    }
	    else if (Month() == date.Month())
	    {
		if (Day() < date.Day())
		{
		    return true;
		}
		else
		{
		    return false;
		}
	    }
	    else
	    {
		return false;
	    }
	}
	else
	{
	    return false;
	}
    }
}

bool CDate::operator>(const CDate &date) const
{
    if (Valid() && !date.Valid())
    {
	return true;
    }
    else if (!Valid())
    {
	return false;
    }
    else
    {
	if (Year() > date.Year())
	{
	    return true;
	}
	else if (Year() == date.Year())
	{
	    if (Month() > date.Month())
	    {
		return true;
	    }
	    else if (Month() == date.Month())
	    {
		if (Day() > date.Day())
		{
		    return true;
		}
		else
		{
		    return false;
		}
	    }
	    else
	    {
		return false;
	    }
	}
	else
	{
	    return false;
	}
    }
}

bool CDate::operator==(const CDate &date) const
{
    if (!Valid() && !date.Valid())
    {
	return true;
    }
    else if (Valid() && date.Valid())
    {
	return (Year()==date.Year() && Month() == date.Month() && Day() == date.Day());
    }
    else
    {
	return false;
    }
}

CDate CDate::Today()
{
	CDate date;

#ifndef WIN32
	time_t Zeit;

    time(&Zeit);

    struct tm *pBrokenTime = localtime(&Zeit);

    date.SetDate(pBrokenTime->tm_mday, pBrokenTime->tm_mon+1, pBrokenTime->tm_year+1900);
#else
	//assert(false);
	//STHTODO
#endif

	return date;
}

CDate CDate::operator+(int nDays) const
{
    CDate date = *this;

    date += nDays;

    return date;
}

void CDate::operator+=(int nDays)
{
    while (nDays > 0)
    {
	int nDaysLeft = m_pDays[Month()-1]-Day();

	if (Month() == 2 && Schaltjahr(Year()))
	{
	    nDaysLeft += 1;
	}

	ASSERT(nDaysLeft >= 0);

	if (nDaysLeft >= nDays)
	{
	    SetDate(Day()+nDays, Month(), Year());

	    ASSERT(Valid());

	    nDays = 0;
	}
	else
	{
	    int nMonth = Month()+1;
	    int nYear = Year();
	    
	    if (nMonth == 13)
	    {
		nMonth = 1;
		nYear++;
	    }
      
	    SetDate(1, nMonth, nYear);
	    
	    nDays -= nDaysLeft+1;
	}
    }
}

void CDate::operator-=(int nDays)
{
    while (nDays > 0)
    {
	if (nDays < Day())
	{
	    SetDate(Day()-nDays, Month(), Year());
	    nDays = 0;
	}
	else
	{
	    if (Day() > 1)
	    {
		nDays -= Day()-1;
		SetDate(1, Month(), Year());
	    }

	    if (Month() == 1)
	    {
		nDays -= 1;
		SetDate(31,12,Year()-1);
	    }
	    else
	    { 
		int nTage = m_pDays[Month()-2];

		if (Month() == 3)
		{
		    nTage = 28;
		    if (Schaltjahr())
		    {
			nTage++;
		    }
		}

		if (nDays <= nTage)
		{
		    SetDate(nTage-nDays+1, Month()-1, Year());
		    nDays = 0;
		}
		else
		{
		    nDays -= nTage;
		    ASSERT(Day() == 1);
		    SetDate(1, Month()-1, Year());
		}
	    }
	}
    }
}

bool CDate::operator!=(const CDate &Date) const
{
    return !operator==(Date);
}

CDate::WEEKDAY CDate::DayOfWeek() const
{
    CDate ref;
    WEEKDAY WeekDay = SUNDAY;

    ref.SetDate(7,10,2001);
    int nDays=0;

    if (ref < *this)
    {
	nDays = (*this-ref)%7;
    }
    else if (ref == *this)
    {
	nDays = 0;
    }
    else
    {
	nDays = (ref-*this)%7;
    }

    switch(nDays)
    {
	case 0:
	    WeekDay = SUNDAY;
	    break;
	case 1:
	    WeekDay = MONDAY;
	    break;
	case 2:
	    WeekDay = TUESDAY;
	    break;
	case 3:
	    WeekDay = WEDNESDAY;
	    break;
	case 4:
	    WeekDay = THURSDAY;
	    break;
	case 5:
	    WeekDay = FRIDAY;
	    break;
	case 6:
	    WeekDay = SATURDAY;
	    break;
    }

    return WeekDay;
}

int CDate::operator-(const CDate &date) const
{
    CDate date1 = *this;
    CDate date2 = date;
    bool bSign=false;

    if (date1 == date2)
    {
	return 0;
    }
    else if (date1<date2)
    {
	date1 = date;
	date2 = *this;
	bSign = true;
    }

    int nDays = 0;
    
    nDays += date1.Day()-1;
    nDays -= date2.Day()-1;
    date1.SetDate(1, date1.Month(), date1.Year());
    date2.SetDate(1, date2.Month(), date2.Year());

    while (date1.Year() > date2.Year())
    {
	if (date1.Month() > 2)
	{
	    if (date1.Schaltjahr())
	    {
		nDays += 366;
	    }
	    else
	    {
		nDays += 365;
	    }
	}
	else
	{
	    CDate refdate;
	    refdate.SetDate(date1.Day(), date1.Month(), date1.Year()-1);

	    if (refdate.Schaltjahr())
	    {
		nDays += 366;
	    }
	    else
	    {
		nDays += 365;
	    }
	}

	date1.SetDate(date1.Day(), date1.Month(), date1.Year()-1);
    }
    
    if (date1 > date2)
    {
	while (date1.Month() > date2.Month())
	{
	    nDays += m_pDays[date1.Month()-2];
	    date1.SetDate(date1.Day(), date1.Month()-1, date1.Year());
	}
    }
    else
    {
	while (date1.Month() < date2.Month())
	{
	    nDays -= m_pDays[date1.Month()-1];
	    date1.SetDate(date1.Day(), date1.Month()+1, date1.Year());
	}
    }

    ASSERT(date1 == date2);

    if (bSign)
    {
	nDays = -nDays;
    }

    return nDays;
}

CStrings CDate::DayName() const
{
    CStrings strName;

    switch(DayOfWeek())
    {
	case SUNDAY:
	    strName = LANGUAGE_STRING(IDS_SUNDAY);
	    break;
	case MONDAY:
	    strName = LANGUAGE_STRING(IDS_MONDAY);    
	    break;
	case TUESDAY:
	    strName = LANGUAGE_STRING(IDS_TUESDAY);    
	    break;
	case WEDNESDAY:
	    strName = LANGUAGE_STRING(IDS_WEDNESDAY);
	    break;
	case THURSDAY:
	    strName = LANGUAGE_STRING(IDS_THURSDAY);
	    break;
	case FRIDAY:
	    strName = LANGUAGE_STRING(IDS_FRIDAY);
	    break;
	case SATURDAY:
	    strName = LANGUAGE_STRING(IDS_SATURDAY);
	    break;
    }

    return strName;
}

CStrings CDate::DayNameShort() const
{
    CStrings strName;

    switch(DayOfWeek())
    {
	case SUNDAY:
	    strName = LANGUAGE_STRING(IDS_SUNDAY_SHORT);
	    break;
	case MONDAY:
	    strName = LANGUAGE_STRING(IDS_MONDAY_SHORT);    
	    break;
	case TUESDAY:
	    strName = LANGUAGE_STRING(IDS_TUESDAY_SHORT);    
	    break;
	case WEDNESDAY:
	    strName = LANGUAGE_STRING(IDS_WEDNESDAY_SHORT);
	    break;
	case THURSDAY:
	    strName = LANGUAGE_STRING(IDS_THURSDAY_SHORT);
	    break;
	case FRIDAY:
	    strName = LANGUAGE_STRING(IDS_FRIDAY_SHORT);
	    break;
	case SATURDAY:
	    strName = LANGUAGE_STRING(IDS_SATURDAY_SHORT);
	    break;
    }

    return strName;
}

bool CDate::operator<=(const CDate &date) const
{
    return (date > *this)||(date == *this);
}

bool CDate::operator>=(const CDate &date) const
{
    return (date < *this)||(date == *this);
}

CStrings CDate::MonthName() const
{
    CStrings strName;

    switch(Month())
    {
	case 1:
	    strName = LANGUAGE_STRING(IDS_JANUARY);
	    break;
	case 2:
	    strName = LANGUAGE_STRING(IDS_FEBRUARY);
	    break;
	case 3:
	    strName = LANGUAGE_STRING(IDS_MARCH);
	    break;
	case 4:
	    strName = LANGUAGE_STRING(IDS_APRIL);
	    break;
	case 5:
	    strName = LANGUAGE_STRING(IDS_MAY);
	    break;
	case 6:
	    strName = LANGUAGE_STRING(IDS_JUNE);
	    break;
	case 7:
	    strName = LANGUAGE_STRING(IDS_JULY);
	    break;
	case 8:
	    strName = LANGUAGE_STRING(IDS_AUGUST);
	    break;
	case 9:
	    strName = LANGUAGE_STRING(IDS_SEPTEMBER);
	    break;
	case 10:
	    strName = LANGUAGE_STRING(IDS_OCTOBER);
	    break;
	case 11:
	    strName = LANGUAGE_STRING(IDS_NOVEMBER);
	    break;
	case 12:
	    strName = LANGUAGE_STRING(IDS_DECEMBER);
	    break;
    }

    return strName;
}

bool CDate::Holiday() const
{
    return NewYear()||
	GoodFriday()||
	Easter()||
	AscensionDay()||
	Whitsuntide()||
	Christmas()||
	NationalFeiertag()||
	EasterMonday()||
	WhitsuntideMonday()||
	StephansDay();
}

bool CDate::NewYear() const
{
    return ((Day() == 1) && (Month() == 1));
}

bool CDate::GoodFriday() const // Karfreitag
{
    return (*this == EasterDate(Year())-2);
}

bool CDate::Easter() const
{
    CDate date = EasterDate(Year());

    return (*this == date);
}

bool CDate::EasterMonday() const
{
    CDate date = EasterDate(Year());

    return (*this == date+1);
}

CDate CDate::EasterDate(int nYear) const
{
    CDate date;
    date.SetDate(1,3,nYear);

    int a = nYear % 19;
    int b = nYear % 4;
    int c = nYear % 7;
    int d = (((nYear/100)*8)+13)/25-2;
    int e = (nYear/100) - (Year()/400) - 2;
    int f = (15 + e - d) % 30;
    int g = (6 + e) % 7;
    int h = (19*a+f) % 30;
    int i = h;
    if (h == 29)
    {
	i = 28;
    }
    if ((h==28)&&(a>10))
    {
	i = 27;
    }
    int j = (2*b+4*c+6*i+g)%7;

    date += i + j + 22 - 1;

    return date;
}

bool CDate::AscensionDay() const // Auffahrt
{
    return (*this == EasterDate(Year())+39);
}

bool CDate::Whitsuntide() const //Pfingsten
{
    CDate date = EasterDate(Year());

    return (*this == (date + 49));
}

bool CDate::WhitsuntideMonday() const //Pfingstmontag
{
    CDate date = EasterDate(Year());

    return (*this == (date + 50));
}

bool CDate::Christmas() const
{
    return ((Month() == 12) && (Day() == 25));
}

bool CDate::StephansDay() const
{
    return ((Month() == 12) && (Day() == 26));
}

/*
function OSTERN_GAUSS(jahr : integer) : integer;
  { Ostern faellt auf den n. Maerz.  Beisp.: OSTERN=33 bedeutet 2.April    }
  { Quelle: MONOID Mathematik-Blatt Karolinen-Gymn. Frankenthal Maerz 1992 }
  { Nach der Gauss-Formel aus dem Jahre 1800. Gueltig fuer 1583 bis 8702   }
  var a,b,c,d,e,f,g,h,i,j : integer;  
  begin
    a := jahr mod 19;
    b := jahr mod 4;
    c := jahr mod 7;
    d := ( ( (jahr div 100) * 8 ) + 13 ) div 25 - 2;
    e := (jahr div 100) - (jahr div 400) - 2;
    f := (15 + e - d) mod 30;
    g := (6 + e) mod 7;
    h := (19 * a + f) mod 30;
    i := h;
    if (h = 29) then i := 28;
    if ( (h = 28) and (a > 10) ) then i := 27;
    j := ( (2 * b) + (4 * c) + (6 * i) + g ) mod 7;
    OSTERN_GAUSS := i + j + 22;
    end;
*/

//pfingsten j = (ostern j + 49)

//christiHim j = (ostern j + 39)

//karfreitag = ostern - 2

CDate CDate::operator-(int nDays) const
{
    CDate date = *this;

    date -= nDays;

    return date;
}

bool CDate::NationalFeiertag() const
{
    return ((Day() == 1) && (Month() == 8));
}

CStrings CDate::HolidayName() const
{
    CStrings strName;

    if (NewYear())
    {
	strName = LANGUAGE_STRING(IDS_NEWYEAR);
    }
    else if (GoodFriday())
    {
	strName = LANGUAGE_STRING(IDS_GOODFRIDAY);
    }
    else if (Easter())
    {
	strName = LANGUAGE_STRING(IDS_EASTER);
    }
    else if (AscensionDay())
    {
	strName = LANGUAGE_STRING(IDS_ASCENSIONDAY);
    }
    else if (Whitsuntide())
    {
	strName = LANGUAGE_STRING(IDS_WHITSUNTIDE);
    }
    else if (Christmas())
    {
	strName = LANGUAGE_STRING(IDS_CHRISTMAS);
    }
    else if (NationalFeiertag())
    {
	strName = LANGUAGE_STRING(IDS_NATIONALFEIERTAG);
    }
    else if (EasterMonday())
    {
	strName = LANGUAGE_STRING(IDS_EASTERMONDAY);
    }
    else if (WhitsuntideMonday())
    {
	strName = LANGUAGE_STRING(IDS_WHITSUNTIDEMONDAY);
    }
    else if (StephansDay())
    {
	strName = LANGUAGE_STRING(IDS_STEPHANSDAY);
    }

    return strName;
}






