/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef DATE_H

#define DATE_H

#include <stdlib.h>
#include <stdio.h>

// Thanks to Gene Heskett from WV, USA for the following fix (seems to solve a compilation problem on rh Linux):
#include <time.h>
typedef const char * PCCHAR;

typedef const char * PCHAR;

/*!
 *
 * \class CDate
 *
 * \author STH
 * \date 26.3.00
 *
 * \brief contains a date
 *
 * earliest date possible is 1.1.1970.
 *
 */

class CStrings;

class CDate
{
 private:
    unsigned int m_nDay:5;
    unsigned int m_nMonth:4;
    unsigned int m_nYear:10;
    unsigned int m_nValid:1;
    //char *m_cBuffer;

    static const int m_pDays[];
 private:
    void SetValid(bool bValid=true);
    bool Schaltjahr(int nYear) const;
 public:
    enum WEEKDAY {SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY};
    
    inline bool Schaltjahr() const { return Schaltjahr(Year());}
    
    inline int Day() const { return (m_nDay&0x1F)+1;}
    inline int Month() const { return (m_nMonth&0xF)+1;}
    inline int Year() const { return (m_nYear&0x3FF)+1970;}
    inline bool Valid() const { return (m_nValid&0x1); }
    
    void SetDate(int nDay, int nMonth, int nYear);
    CDate();
    CDate(const CDate &date);
    CDate(const char *strDate);
    ~CDate();
    
    //operator PCHAR();
    operator CStrings() const;
    void operator=(const char *cDatum);
    void operator=(const CDate &date);
    bool operator<(const CDate &date) const;
    bool operator>(const CDate &date) const;
    bool operator==(const CDate &date) const;
    bool operator!=(const CDate &date) const;
    bool operator<=(const CDate &date) const;
    bool operator>=(const CDate &date) const;

    static CDate Today();
    
    CDate operator+(int nDays) const;
    int operator-(const CDate &date) const;
    void operator+=(int nDays);
    void operator-=(int nDays);
    CDate operator-(int nDays) const;

    WEEKDAY DayOfWeek() const;
    
    CStrings DayName() const;
    CStrings DayNameShort() const;
    CStrings MonthName() const;

    bool Holiday() const;
    bool NewYear() const;
    bool GoodFriday() const; // Karfreitag
    bool Easter() const;
    bool EasterMonday() const;
    bool AscensionDay() const; // Auffahrt
    bool Whitsuntide() const; //Pfingsten
    bool Christmas() const;
    CDate EasterDate(int nYear) const;
    bool NationalFeiertag() const;
    CStrings HolidayName() const;
    bool WhitsuntideMonday() const;
    bool StephansDay() const;
};

#endif







