/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "file.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "macrodefinitions.h"
#include <errno.h>
#include "stringtokenizer.h"
#ifdef _UNIX
#include <dirent.h>
#endif

CFile::CFile()
{
    m_deskriptor = NULL;
    m_bPrivateDeskriptor = true;
}

CFile::CFile(FILE *deskriptor)
{
    m_deskriptor = deskriptor;
    m_bPrivateDeskriptor = false;
}

CFile::~CFile()
{
    if (m_bPrivateDeskriptor)
    {
	Close();
    }
}

bool CFile::Exist(const char *strName)
{
    /*struct stat stats;

      return !stat(strName, &stats);*/

#ifndef WIN32
	if (!access(strName, F_OK))
    {
	return true;
    }
    else if (errno == ENOENT)
    {
	return false;
    }
    else
    {
	return true; // File exists, but is not accessible
    }
#else
	//assert(false);
	//STHTODO
	return false;
#endif
}

bool CFile::Open(const char *strName, const char *strMode)
{
    if (m_bPrivateDeskriptor)
    {
	Close();
    }

    m_bPrivateDeskriptor = true;
    m_deskriptor = fopen(strName, strMode);
    m_strName = strName;
    m_strMode = strMode;

    return IsOpen();
}

bool CFile::IsOpen()
{
    return (m_deskriptor != NULL);
}

void CFile::Close()
{
    ASSERT(m_bPrivateDeskriptor);

    if (IsOpen())
    {
	fclose(m_deskriptor);
	m_deskriptor = NULL;
    }
}

const CStrings &CFile::Name() const
{
    return m_strName;
}

void CFile::Flush()
{
    if (IsOpen())
    {
	fflush(m_deskriptor);
    }
}

bool CFile::Remove(const char *strName)
{
    return (!remove (strName));
}

bool CFile::IsDir(const char *strPath)
{
#ifndef WIN32
	CStrings strEntity = strPath;

    ASSERT(strEntity[0] == '/');

    if (strEntity == "/")
    {
	return true;
    }
    else
    {
	/*CString strTopDir = TopDir(strEntity);

	  ASSERT(strTopDir != "");

	  DIR *pDir = opendir(strTopDir);

	  if (pDir != NULL)
	  {
	  while (dirent *pDirEnt = readdir(pDir))
	  {
	  if (strEntity == pDirEnt->d_name)
	  {*/
	struct stat buffer;
			
	stat(strEntity, &buffer);
	//closedir(pDir);
			
	if (S_ISDIR(buffer.st_mode))
	{
	    return true;
	}
	else
	{
	    return false;
	}
	/*}
	  }	

	  closedir(pDir);
	  }*/

	return false;
    }
#else
	//assert(false);
	//STHTODO
	return false;
#endif
}

bool CFile::mkdir(const char *strPath, bool bCreateWholePath)
{
#ifndef WIN32
	if (Exist(strPath))
    {
	if (IsDir(strPath))
	{
	    return true;
	}
	else
	{
	    return false;
	}
    }
    else
    {
	if (!bCreateWholePath)
	{
	    if (mkdir(strPath,0x1FF) == 0)
	    {
		return true;
	    }
	    else
	    {
		return false;
	    }
	}
	else
	{
	    CStringTokenizer Tokenizer("/");

	    Tokenizer << strPath;

	    CStrings strToken;
	    CStrings strIncPath;

	    while (Tokenizer.NextToken(strToken))
	    {
		strIncPath += strToken+"/";

		if (!Exist(strIncPath))
		{
		    if (::mkdir(strIncPath, 0x1F8))
		    {
			return false;
		    }
		    
		}
		else
		{
		    if (!IsDir(strIncPath))
		    {
			return false;
		    }
		}
	    }

	    return true;
	}
    }
#else
	//assert(false);
	//STHTODO
	return false;
#endif
}

CStrings CFile::TopDir(const char *strPath)
{
    CStrings strEntity = strPath;

    if (strEntity.Right(1) == "/")
    {
	strEntity = strEntity.Left(strEntity.Length()-1);
    }

    int nIndex = strEntity.FindReverse('/');
    if (nIndex >= 0)
    {
	strEntity = strEntity.Left(nIndex+1);
    }
    else
    {
	strEntity = "";
    }

    return strEntity;
}

