/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "messageclient.h"
#include "simpleiterator.h"

CMessageClient::CMessageClient(int nId)
{
    m_nId = nId;
}

CMessageClient::~CMessageClient()
{
    UnHookAll();
}

void CMessageClient::UnHookAll()
{
    while ( CMessageListener *pElement = m_ListenTo[0] )
    {
	UnHook(pElement->Client());
    }
	
    while ( CMessageListener *pElement = m_ListenedBy[0] )
    {
	pElement->Client()->UnHook(this);
    }
}

/*void CMessageClient::OnMessage(CMessageClient *pClient, unsigned long lMask)
{

}*/

void CMessageClient::OnMessage(CMessageClient *pClient, unsigned long lMask, void *pData)
{

}

void CMessageClient::ListenTo(CMessageClient *pClient, unsigned long lMask)
{
    CMessageListener *pML = NULL;
    CSimpleIterator<CMessageListener *> ML_It;
	
    ML_It << &m_ListenTo;
	
    bool bFound = false;
	
    while ( !bFound && (pML = ML_It++) )
    {
	if ( pML->Client() == pClient )
	{
	    bFound = true;
	}
    }
	
    if ( pML == NULL )
    {
	pML = new CMessageListener(pClient,lMask);
		
	m_ListenTo += pML;
    }
    else
    {
	pML->m_lMask |= lMask;
    }
	
    ML_It << &(pClient->m_ListenedBy);
	
    bFound = false;
    pML = NULL;
	
    while ( !bFound && (pML = ML_It++) )
    {
	if ( pML->Client() == this )
	{
	    bFound = true;
	}
    }
	
    if ( pML == NULL )
    {
	pML = new CMessageListener(this,lMask);
		
	pClient->m_ListenedBy += pML;
    }
    else
    {
	pML->m_lMask |= lMask;
    }
}

CMessageClient::CMessageListener::CMessageListener(CMessageClient *pElement, unsigned long lMask)
{
    m_pClient = pElement;
    m_lMask = lMask;
}

/*void CMessageClient::SendMessage(unsigned long lMask)
{
    CSimpleIterator<CMessageListener *> ML_It;
	
    ML_It << &m_ListenedBy;
	
    while ( CMessageListener *pML = ML_It++ )
    {
	if ( unsigned long lAndMask = pML->Mask() & lMask )
	{
	    pML->Client()->OnMessage(this,lAndMask);
	}
    }
}*/

void CMessageClient::Notify(int nNotification, void *pData)
{
    notification_data data = {nNotification, pData};

    SendMessage(EVENT_MASK_NOTIFY, &data);
}

void CMessageClient::SendMessage(unsigned long lMask, void *pData)
{
    CSimpleIterator<CMessageListener *> ML_It;
	
    ML_It << &m_ListenedBy;
	
    while ( CMessageListener *pML = ML_It++ )
    {
	if ( unsigned long lAndMask = pML->Mask() & lMask )
	{
	    pML->Client()->OnMessage(this, lAndMask, pData);
	}
    }
}

void CMessageClient::UnHook(CMessageClient *pClient, unsigned long lMask)
{
//     CSimpleIterator<CMessageListener *> ML_It;
    unsigned long lCompMask = ~lMask;
		
//     ML_It << &(pClient->m_ListenedBy);
	
    bool bFound = false;
    CMessageListener *pML=NULL;

    int nIndex = 0;

    while (!bFound && (nIndex < pClient->m_ListenedBy.card()))
    {
	pML = pClient->m_ListenedBy[nIndex];

	if ( pML->Client() == this )
	{
	    bFound = true;
	    pML->m_lMask &= lCompMask;
			
	    if ( pML->Mask() == 0L )
	    {
		pClient->m_ListenedBy -= pML;
		delete pML;
	    }
	}

	nIndex++;
    }
	
//     ML_It << &m_ListenTo;
	
    bFound = false;
    pML = NULL;

    nIndex = 0;

    while ( !bFound && (nIndex < m_ListenTo.card()) )
    {
// 	pML = ML_It++;
	
	pML = m_ListenTo[nIndex];

	if (pML->Client() == pClient)
	{
	    bFound = true;
	    pML->m_lMask &= lCompMask;
			
	    if ( pML->Mask() == 0L )
	    {
		m_ListenTo -= pML;
		delete pML;
	    }
	}

	nIndex++;
    }
}
