/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef MESSAGECLIENT_H

#define MESSAGECLIENT_H

#define EVENT_MASK_VALUE_UPDATE      0x00000001L
#define EVENT_MASK_BUTTON_PUSHED     0x00000002L
#define EVENT_MASK_SELECTION_CHANGED 0x00000004L
#define EVENT_MASK_NEW_ADDED         0x00000008L
#define EVENT_MASK_CHECK_CHANGED     0x00000010L
#define EVENT_MASK_VALUE_CHANGED     0x00000020L
#define EVENT_MASK_ENTER             0x00000040L
#define EVENT_MASK_LEAVE             0x00000080L
#define EVENT_MASK_NOTIFY            0x80000000L
#define EVENT_MASK_RESIZE            0x00000100L

#include "simplelist.h"

struct notification_data
{
    int nNotification;
    void *pData;
};

class CMessageClient
{
    class CMessageListener
	{
	    friend class CMessageClient;
	private:
	    CMessageClient *m_pClient;
	    unsigned long m_lMask;
	public:
	    CMessageListener(CMessageClient *pClient, unsigned long lMask);
			
	    inline CMessageClient *Client() { return m_pClient; }
	    inline unsigned long Mask() { return m_lMask; }
	};
 private:
    CSimpleList<CMessageListener *> m_ListenTo;
    CSimpleList<CMessageListener *> m_ListenedBy;
    int m_nId;
 private:
    void UnHookAll();
 protected:
    //virtual void OnMessage(CMessageClient *pClient, unsigned long lMask);	
    virtual void OnMessage(CMessageClient *pClient, unsigned long lMask, void *pData=NULL);	
 public:
    void ListenTo(CMessageClient *pElement, unsigned long lMask=0xFFFFFFFF);
    //void SendMessage(unsigned long lMask);	
    void SendMessage(unsigned long lMask, void *pData=NULL);	
    void UnHook(CMessageClient *pClient, unsigned long lMask=0xFFFFFFFF);
    void Notify(int nNotification, void *pData=NULL);

    CMessageClient(int nId = 0);
    virtual ~CMessageClient();

    inline int Id() const { return m_nId; }
};

#endif
