/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include <stdio.h>
#include "propertiesfile.h"
#include "macrodefinitions.h"
#include "simpleiterator.h"
#include "stdiofile.h"

CPropertiesFile::CPropertiesFile(const char *strName)
{
    m_strFileName = strName;

    LoadValues();
}

CPropertiesFile::CPropertiesFile()
{
}

CPropertiesFile::~CPropertiesFile()
{
    CStrings strFileName = FileName();
	
    FILE *deskriptor = fopen((const char *)strFileName, "w");
	
    ASSERT(deskriptor != NULL);
	
    CSimpleIterator<CPropertiesFileEntry *> EntryIt;
	
    EntryIt << &m_EntryList;
	
    while (CPropertiesFileEntry *pEntry = EntryIt++)
    {
	if (pEntry->Value() != "")
	{
	    CStrings strLine = pEntry->Name() + " = " + pEntry->Value()+"\n";
	    
	    fputs((const char *)strLine, deskriptor);	
	}

	delete pEntry;
    }

    fclose(deskriptor);
}

CStrings CPropertiesFile::FileName(bool bHome)
{
    if (bHome)
    {
	if (m_strFileName[0] != '/')
	{
	    CStrings strHome = getenv("HOME");
	
	    ASSERT(strHome != "");
	
	    CStrings strFileName = strHome + "/" + m_strFileName;
	
	    return strFileName;
	}
	else
	{
	    return m_strFileName;
	}
    }
    else
    {
	return m_strFileName;
    }
}

void CPropertiesFile::LoadValues(bool bHome)
{
    ASSERT(m_EntryList.card() == 0);
	
    CStrings strFileName = FileName(bHome);
	
    //FILE *deskriptor = fopen((const char *)strFileName, "r");
    CStdioFile deskriptor;

    if (deskriptor.Open((const char *)strFileName, "r"))
    {
	CStrings strLine;
		
	while (deskriptor.ReadLine(strLine,true))
	{
	    int nIndex = strLine.Find('=');
			
	    ASSERT(nIndex >= 0);
			
	    CStrings strEntryName = strLine.Left(nIndex);
	    strEntryName.Trim();
			
	    CStrings strValue = strLine.Mid(nIndex+1);
	    strValue.Trim();
			
	    m_EntryList.AddEntry((const char *)strEntryName, (const char *)strValue);			
	}
		
	deskriptor.Close();
    }
}

void CPropertiesFile::CPropertiesFileList::AddEntry(const char *strEntryName, const char *strValue)
{
    CPropertiesFileEntry *pEntry = FindEntry(strEntryName);
	
    if (pEntry == NULL)
    {
	pEntry = new CPropertiesFileEntry;
	pEntry->Name() = strEntryName;
		
	*this += pEntry;
    }
	
    pEntry->Value() = strValue;
}

CPropertiesFile::CPropertiesFileEntry *CPropertiesFile::CPropertiesFileList::FindEntry(const char *strEntry)
{
    CPropertiesFileEntry *pEntry = NULL;


    for (int nIndex = 0; nIndex < card(); nIndex++)
    {
	pEntry = (*this)[nIndex];

	if (pEntry == NULL)
	{
	}
	if (strEntry == NULL)
	{
	}

	if (pEntry->Name() == strEntry)
	{
	    return pEntry;
	}

    }
	
    return NULL;
}

CStrings CPropertiesFile::GetEntry(const char *strEntry)
{
    if (CPropertiesFileEntry *pEntry = m_EntryList.FindEntry(strEntry))
    {
	CStrings strValue = pEntry->Value();
		
	return strValue;
    }
    else
    {
	CStrings strValue = "";
		
	return strValue;
    }
}

void CPropertiesFile::SetEntry(const char *strEntry, const char *strValue)
{
    m_EntryList.AddEntry(strEntry, strValue);
}

void CPropertiesFile::SetEntry(const char *strEntry, int nValue)
{
    CStrings strData;
    strData.Format("%d", nValue);
    SetEntry(strEntry, (const char *)strData);
}

void CPropertiesFile::SetEntry(const char *strEntry, const CDate &date)
{
    CStrings strData = (CStrings)date;

    SetEntry(strEntry, (const char *)strData);
}
