/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef SIMPLEITERATOR_H

#define SIMPLEITERATOR_H

#include "simplelist.h"
#include "simpleobjectlist.h"

template <class node>
class CSimpleIterator 
{
 protected:
    int counter;
    int m_nPoppedCounter;
    CSimpleList<node> const *list_pointer;
 public:
    CSimpleIterator();
    node operator++(int);
    node operator++();
    node operator--(int);
    node operator--();
    node operator*();
    void operator<<(CSimpleList<node> const *L);
    void reset();
    inline int more_elements() { return(counter<list_pointer->card());}
    void PushPosition();
    void PopPosition();
    inline CSimpleList<node> const *ListPointer() const { return list_pointer; }
};

//---------------------------------------------------------------------------------------
// Implementation Area
//---------------------------------------------------------------------------------------

template <class node>
void CSimpleIterator<node>::PushPosition()
{
    // CAUTION: the stacksize is 1!

    m_nPoppedCounter = counter;
}

template <class node>
void CSimpleIterator<node>::PopPosition()
{
    // CAUTION: the stacksize is 1!

    counter = m_nPoppedCounter;
}

template <class node>
CSimpleIterator<node>::CSimpleIterator()
{
    list_pointer = 0;
    m_nPoppedCounter = 0;
}

template <class node>
node CSimpleIterator<node>::operator++(int)
{
    if(counter<list_pointer->card())
    {
	return (*list_pointer)[counter++];
    }
    else
    {
	return (node)0;
    }
}

template <class node>
node CSimpleIterator<node>::operator++()
{
    if ( counter<list_pointer->card()-1 && counter >= 0)
    {
	return (*list_pointer)[++counter];		
    }
    else
    {
	return NULL;
    }
}

template <class node>
node CSimpleIterator<node>::operator--(int)
{
    if(counter>=0)
    {
	return (*list_pointer)[counter--];
    }
    else
    {
	return (node)0;
    }      
}

template <class node>
node CSimpleIterator<node>::operator--()
{
    if ( counter >= 1 )
    {
	return (*list_pointer)[--counter];
    }
    else
    {
	return NULL;
    }
}

template <class node>
node CSimpleIterator<node>::operator*()
{
    if(counter<list_pointer->card())
    {
	return (*list_pointer)[counter];
    }
    else
    {
	return (node)0;
    }      
}

template <class node>
void CSimpleIterator<node>::operator<<(CSimpleList<node> const *L)
{
    list_pointer = L;
    counter = 0;
}

template <class node>
void CSimpleIterator<node>::reset()
{
    counter = 0;
}

#endif




