/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef SIMPLELIST_H

#define SIMPLELIST_H

#include <stdlib.h>
#include "macrodefinitions.h"

/*!
*
* \class CSimpleList
*
* \author STH
* \date 2.6.00
*
*/

template <class node>
class CSimpleList
{
 private:
    node *field;
    int current_number_of_elements;
    int max_number_of_elements;
 private:
    void ExpandMemory();
    void ExpandMemory(int nGrowBy);
    int StandardGrowBy();
 protected:
    inline void ReduceCardBy1() { current_number_of_elements--; }
 public:
    inline node *get_field() const { return field;}
    CSimpleList(unsigned int maxnumber = 10);
    ~CSimpleList();

    void operator+=(node x);
    int operator-=(node x);
    void operator=(const CSimpleList<node> &List);

    void reset();
    int card() const;
    virtual node in(node x) const;
    node last() const;
    node first() const;

    void flip();
    void get_members(CSimpleList<node> const *l);
    node operator[](int nIndex) const;

    void InsertAt(int nPosition, node Data);
    void RemoveAt(int nPosition);
};

//---------------------------------------------------------------------------------------
// Implementation Area
//---------------------------------------------------------------------------------------

template <class node>
int CSimpleList<node>::card() const
{
    return current_number_of_elements;
}

template <class node>
void CSimpleList<node>::operator=(const CSimpleList<node> &List)
{
    get_members(&List);
}

template <class node>
void CSimpleList<node>::InsertAt(int nPosition, node Data)
{
    if (nPosition >= card())
    {
	*this += Data;
    }
    else
    { 
	if (nPosition < 0)
	{
	    nPosition = 0;
	}

	if (current_number_of_elements >= max_number_of_elements)
	{
	    ExpandMemory();
	} 

	node *pZeiger = field+card();

	for (int nIndex = card(); nIndex > nPosition; nIndex--)
	{
	    *pZeiger = *(pZeiger-1);
	    pZeiger--;
	}

	*pZeiger = Data;
	current_number_of_elements++;
    }
}

template <class node>
void CSimpleList<node>::RemoveAt(int nPosition)
{
    if(nPosition<current_number_of_elements)
    {
	node *zeiger = field+nPosition;

      	node *dest = zeiger;
      	node *source = zeiger+1;
      	nPosition++;

      	while(nPosition<current_number_of_elements)
      	{
	    nPosition++;
	    *dest++ = *source++;
      	}

      	current_number_of_elements--;
    }
}

template <class node>
CSimpleList<node>::CSimpleList(unsigned int maxnumber)
{
    max_number_of_elements = maxnumber;
    current_number_of_elements = 0;
    field = new node[max_number_of_elements];		
}

template <class node>
CSimpleList<node>::~CSimpleList()
{
    delete field;
}

template <class node>
void CSimpleList<node>::ExpandMemory(int nGrowBy)
{
    ASSERT(nGrowBy > max_number_of_elements);

    max_number_of_elements = nGrowBy;
    field = (node *)realloc(field,max_number_of_elements*sizeof(node));    
}

template <class node>
void CSimpleList<node>::ExpandMemory()
{
    int nGrowBy = StandardGrowBy();
    ExpandMemory(max_number_of_elements+nGrowBy+1);
}

template <class node>
int CSimpleList<node>::StandardGrowBy()
{
    return (max_number_of_elements>>3);
}

template <class node>
void CSimpleList<node>::operator+=(node x)
{
    if(current_number_of_elements>=max_number_of_elements)
    {
	ExpandMemory();
    }

    field[current_number_of_elements] = x;
    current_number_of_elements++;
}

template <class node>
int CSimpleList<node>::operator-=(node x)
{
    //node *zeiger,*source,*dest;
    node *zeiger;
    int i;

    zeiger = field;
    i = 0;
    while((i<current_number_of_elements)&&(*zeiger!=x))
    {
      	zeiger++;
      	i++;
    }

    if(i<current_number_of_elements)
    {
/*
      	dest = zeiger;
      	source = zeiger+1;
      	i++;

      	while(i<current_number_of_elements)
      	{
	    i++;
	    *dest++ = *source++;
      	}

      	current_number_of_elements--;
*/
	RemoveAt(i);
      	return 1;
    }
    else
    {
      	return 0;
    }
}

template <class node>
void CSimpleList<node>::reset()
{
    current_number_of_elements = 0;	
}

template <class node>
node CSimpleList<node>::in(node x) const
{
    node *zeiger;
    int i;

    zeiger = field;
    i = 0;
      
    while(i<current_number_of_elements)
    {
	if(*zeiger==x)
	{
	    return(*zeiger);
	}
	i++;
	zeiger++;
    }
      
    return (node)0;
}

template <class node>
node CSimpleList<node>::last() const
{
    if(current_number_of_elements)
    {
	return field[current_number_of_elements-1];
    }
    else
    {
	return 0;
    }
}

template <class node>
node CSimpleList<node>::first() const
{
    if(current_number_of_elements)
    {
	return *field;
    }
    else
    {
	return 0;
    }
}

template <class node>
void CSimpleList<node>::flip()
{
    node *n1, *n2;
    node n;

    if(card())
    {
	n1 = field;
	n2 = field+current_number_of_elements-1;
	while(n2 > n1)
	{
	    n = *n2;
	    *n2 = *n1;
	    *n1 = n;
	    n1++;
	    n2--;
	}
    }
}

template <class node>
void CSimpleList<node>::get_members(CSimpleList<node> const *l)
{
    if(max_number_of_elements<l->card())
    {
	ExpandMemory(l->card()+10);
    }

    current_number_of_elements = l->card();
    if (current_number_of_elements > 0)
    {
	memcpy(field,l->field,current_number_of_elements*sizeof(node));
    }
}

template <class node>
node CSimpleList<node>::operator[](int nIndex) const
{
    if ( nIndex < card() && nIndex >= 0 )
    {
	return field[nIndex];
    }
    else
    {
	return (node)NULL;
    }
}

#endif










