/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef SIMPLEOBJECTLIST_H

#define SIMPLEOBJECTLIST_H

#include "simplelist.h"

template <class object>
class CSimpleObjectList: private CSimpleList<object *>
{
 public:
    inline object **get_field() { return CSimpleList<object *>::get_field();}
    CSimpleObjectList(unsigned int maxnumber = 10);
    CSimpleObjectList(const CSimpleObjectList<object>& List);
    ~CSimpleObjectList();

    void operator+=(const object &x);
    void operator-=(object *);
    void reset();
    inline int card() const { return CSimpleList<object *>::card(); }
    virtual object *in(const object &x) const;
    inline object *last() const { return CSimpleList<object *>::last(); }
    inline object *first() const { return CSimpleList<object *>::first(); }

    inline object *operator[](int nIndex) const { return CSimpleList<object *>::operator[](nIndex); }    
    object *Add(const object &x);
    object *InsertAt(int nIndex, const object &x);

    bool operator==(const CSimpleObjectList<object>& objects) const;
    void operator=(const CSimpleObjectList<object>& objects);
    void operator=(const CSimpleList<object *>& objects);
    void RemoveAt(int nIndex);
};

//---------------------------------------------------------------------------------------
// Implementation Area
//---------------------------------------------------------------------------------------

template <class object>
//CSimpleObjectList<object>::CSimpleObjectList<object>(const CSimpleObjectList<object> &List) 
CSimpleObjectList<object>::CSimpleObjectList(const CSimpleObjectList<object> &List) 
	: CSimpleList<object *>()
{
    *this = List;
}

template <class object>
void CSimpleObjectList<object>::RemoveAt(int nIndex)
{
    object *pData = (*this)[nIndex];
    
    if (pData != NULL)
    {
	*this -= pData;
	delete pData;
    }
}

template <class object>
void CSimpleObjectList<object>::operator=(const CSimpleObjectList<object> &objects)
{
    reset();

    for (int nIndex = 0; nIndex < objects.card(); nIndex++)
    {
	object *pObject = objects[nIndex];

	*this += *pObject;
    }
}

template <class object>
void CSimpleObjectList<object>::operator=(const CSimpleList<object *> &objects)
{
    reset();

    for (int nIndex = 0; nIndex < objects.card(); nIndex++)
    {
	object *pObject = objects[nIndex];

	*this += *pObject;
    }
}

template <class object>
bool CSimpleObjectList<object>::operator==(const CSimpleObjectList<object>& objects) const
{
  if (card() == objects.card())
    {
    for (int nIndex = 0; nIndex < card(); nIndex++)
      {
	object *pObject1 = (*this)[nIndex];

	if (!objects.in(*pObject1))
	  {
	    return false;
	  }
      }
    return true;
  }
  else
    {
    return false;
  }
}

template <class object>
object *CSimpleObjectList<object>::InsertAt(int nIndex, const object &x)
{
    object *pObject = new object(x);
    
    CSimpleList<object *>::InsertAt(nIndex, pObject);

    return pObject;
}

template <class object>
object *CSimpleObjectList<object>::Add(const object &x)
{
    object *pObject = new object(x);
    
    CSimpleList<object *>::operator+=(pObject);

    return pObject;
}

template <class object>
void CSimpleObjectList<object>::operator+=(const object &x)
{
    Add(x);
}

template <class object>
void CSimpleObjectList<object>::operator-=(object *x)
{
    CSimpleList<object *>::operator-=(x);
    
    delete x;
}

template <class object>
object *CSimpleObjectList<object>::in(const object& x) const
{
  for (int nIndex = 0; nIndex < card(); nIndex++)
    {
      object *pObject = (*this)[nIndex];

      if (*pObject == x)
	{
	  return pObject;
	}
    }
  
  return NULL;
}

template <class object>
//CSimpleObjectList<object>::CSimpleObjectList<object>(unsigned int maxnumber)
CSimpleObjectList<object>::CSimpleObjectList(unsigned int maxnumber)
  : CSimpleList<object *>(maxnumber)
{

}

template <class object>
CSimpleObjectList<object>::~CSimpleObjectList()
{
  reset();
}

template <class object>
void CSimpleObjectList<object>::reset()
{
  for (int nIndex = 0; nIndex < card(); nIndex++)
  {
    object *pObject = (*this)[nIndex];

    delete pObject;
  }

  CSimpleList<object *>::reset();
}

#endif

