#ifndef SORTEDITERATOR_H
#define SORTEDITERATOR_H

#include "bufferediterator.h"
#include "filterediterator.h"

template <class element>
class sorted_iterator : public buffered_iterator<element>
{
    friend class filtered_iterator<element>;
 private:
    int (*orderby)(const void *, const void *);

    void order();
 protected:
    virtual void retrieve_elements();
 public:
    sorted_iterator();
    void set_order(int sort_function(const void*, const void*));

    element succ(element pBase);
    element pred(element pBase);
    void unify();
};

//---------------------------------------------------------------------------------------
// Implementation Area
//---------------------------------------------------------------------------------------

template <class element>
void sorted_iterator<element>::unify()
{
    ASSERT(orderby != NULL);

    PushPosition();

    reset();

    element pElement = NULL;

    while (pElement = (*this)++)
    {
	element pNextElement = *(*this);

	if (pNextElement != NULL)
	{
	    if (orderby(&pElement, &pNextElement) == 0)
	    {
		*this -= pNextElement;
		reset();
	    }
	}
    }

    PopPosition();
}

template <class element>
element sorted_iterator<element>::succ(element pBase)
{
    PushPosition();
    reset();

    element pResult = NULL;

    while (pResult = (*this)++)
    {
	if (pResult == pBase)
	{
	    if (*(*this) == NULL)
	    {
		reset();
		pResult = *(*this);
		PopPosition();
		return pResult;
	    }
	    else
	    {
		pResult = *(*this);
		PopPosition();
		return pResult;
	    }
	}
    }
	
    PopPosition();
    return NULL;
}

template <class element>
element sorted_iterator<element>::pred(element pBase)
{
    PushPosition();
    reset();

    element pResult = NULL;
    element pPred = last();

    while (pResult = *(*this))
    {
	if (pResult == pBase)
	{
	    PopPosition();
	    return pPred;
	}
	else
	{
	    pPred = pResult;

	    (*this)++;
	}
    }
	
    PopPosition();
    return NULL;
}
 
template <class element>
void sorted_iterator<element>::order()
{
    int i;

    if(orderby&&internal_field)
    {
	list_get_members(internal_field);

	int nCard = list_cn();

	if (nCard > 0)
	{
	    element *pElement = list_get_field();
	    qsort(pElement,nCard,sizeof(element),orderby);
	}     
    }
    list_touch();
}

template <class element>
void sorted_iterator<element>::retrieve_elements()
{
    order();
}

template <class element>
sorted_iterator<element>::sorted_iterator()
{
    orderby = 0;
}

template <class element>
void sorted_iterator<element>::set_order(int sort_function(const void*, const void*))
{
    orderby = sort_function;
    order();
}

#endif
