/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "stdiofile.h"
#include "macrodefinitions.h"
#include <stdarg.h>

int CStdioFile::m_nBufferSize = 1024;

int CStdioFile::BufferSize()
{
    return m_nBufferSize;
}

CStdioFile::CStdioFile(FILE *deskriptor)
    : CFile(deskriptor)
{
    Initialize();
}

CStdioFile::CStdioFile()
{
    Initialize();
}

void CStdioFile::Initialize()
{
    m_cBuffer = new char[BufferSize()];
}

CStdioFile::~CStdioFile()
{
    delete m_cBuffer;
}

bool CStdioFile::WriteLine(const char *strData)
{
    if (IsOpen())
    {
	fputs(strData, Deskriptor());

	return true;
    }
    else
    {
	return false;
    }
}

bool CStdioFile::ReadLine(CStrings &strData, bool bClipNewLine)
{
    if (IsOpen())
    {
	bool bErfolg = false;
	
	ASSERT(Deskriptor() != NULL);
	
	strData.Empty();
		
	if (!feof(Deskriptor()))
	{
	    bool bEndReached = false;
		
	    while (!bEndReached && (fgets(Buffer(), BufferSize(), Deskriptor()) != NULL))
	    {
		bErfolg = true;
			
		strData += Buffer();
			
		if (strData[strData.Length()-1] == '\n')
		{
		    bEndReached = true;
				
		    if (bClipNewLine)
		    {
			strData.RightTrim('\n');
		    }
		}		
	    }
	}
	
	return bErfolg;

    }
    else
    {
	return false;
    }
}

bool CStdioFile::WriteFormatLine(const char *cFormat, ...)
{
    if (IsOpen())
    {
	va_list argptr;
	va_start(argptr,cFormat);
	
	vfprintf(Deskriptor(), cFormat, argptr);
	
	va_end(argptr);
	return true;
    }
    else
    {
	return false;
    }
}

void CStdioFile::operator=(const CStdioFile &File)
{
    Close();
    Open(File.Name(), "r+");
    long lPos = ftell(File.Deskriptor());
    fseek(Deskriptor(), lPos, SEEK_SET);
}


