/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "stringtokenizer.h"

CStringTokenizer::CStringTokenizer(const char *strDelimiter)
{
    m_strDelimiter = strDelimiter;
}

bool CStringTokenizer::NextToken(CStrings &strToken, bool bIncludeToken)
{
    if (m_nPosition >= m_strString.Length())
    {
	return false;
    }
    else if (IsDelimiter(m_strString[m_nPosition]))
    {
	if (!bIncludeToken)
	{
	    strToken = "";
	}
	else
	{
	    strToken = m_strString[m_nPosition];
	}

	m_nPosition++;

	return true;
    }
    else
    {
	int nPosition = m_nPosition+1;

	while ((nPosition<m_strString.Length())&&!IsDelimiter(m_strString[nPosition]))
	{
	    nPosition++;
	}

	int nEnd = nPosition-1;

	if (bIncludeToken)
	{
	    if (nPosition < m_strString.Length())
	    {
		nEnd++;
	    }
	}

	strToken = m_strString.Mid(m_nPosition, nEnd);
	
	m_nPosition = nPosition+1;

	return true;
    }
}

bool CStringTokenizer::IsDelimiter(char cTest)
{
    return (strchr(m_strDelimiter, cTest) != NULL);
}

void CStringTokenizer::operator<<(const char *strString)
{
    m_strString = strString;
    m_nPosition = 0;
}
