/*******************************************************************************
 *
 * libbasic Version 0.0.1
 *
 * libbasic is a simple framework of foundation classes written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef TABLE_H

#define TABLE_H

#include "unique.h"
#include "date.h"
#include "stdiofile.h"
#include "record.h"

class CRecord;

class CTable: private unique_list<CRecord *>, private CStdioFile
{
 private:
    int m_nCurrentIndex;
    long m_lTop;
    int m_nTableType;
    //long m_lFirstGap;
    CStrings m_strRoot;
    bool m_bCached;
    bool m_bWritten;
    CRecord m_Record;
 public:
    enum TABLESTATUS {TS_NOTFOUND=-5, TS_NOTABLE=-4, TS_NOTOPEN=-3, TS_EOF=-2, TS_BOF=-1, TS_OK=0};
 private:
    CStrings FullPath(const char *strPath);
    bool ReadHeader();
    inline void SetTableType(int nTableType) { m_nTableType = nTableType; }
    /*inline void SetFirstGap(long lFirstGap) { m_lFirstGap = lFirstGap; }
      inline long FirstGap() { return m_lFirstGap; }*/
    void Initialize();
    bool IsTableType(CStrings &strLine);
    //bool IsFirstGap(CStrings &strLine);
    //long FirstGap(CStrings &strLine);
    int TableType(CStrings &strLine);

    inline int CurrentIndex() const { return m_nCurrentIndex; }
    void SetCurrentIndex(int nCurrentIndex);
    inline bool Cached() const { return m_bCached; }
    inline void SetCached(bool bCached=true) { m_bCached = bCached; }
    void Cache();
    inline bool Written() const { return m_bWritten; }
    inline void SetWritten(bool bWritten = true) { m_bWritten = bWritten; }

    void SkipRecord(bool bForward);
    void WriteBack();
    void WriteLine(const char *strLine);
    bool RecordFits(CRecord &m_Record);
 public:
    CTable(const char *strRoot);
    CTable();
    CTable(const CTable &Table);
    ~CTable();

    int CurrentId();
    int SecondId();

    TABLESTATUS Skip(int nSkip=1);
    bool Eof();
    bool Bof();
    TABLESTATUS Set(int nId, double dData);
    TABLESTATUS Set(int nId, const char *strData);
    TABLESTATUS Set(int nId, int nData);
    TABLESTATUS Set(int nId, const CDate &dtData);
    inline TABLESTATUS Set(int nId, unsigned int nData) { return Set(nId, (int)nData); }
    inline TABLESTATUS Set(int nId, IDENT nData) { return Set(nId, (int)nData); }
    inline TABLESTATUS Set(int nId, bool bData) { return Set(nId, (int)bData); }

    TABLESTATUS Get(int nId, double &dData);
    TABLESTATUS Get(int nId, CStrings &strData);
    TABLESTATUS Get(int nId, int &nData);
    TABLESTATUS Get(int nId, CDate &dtData);
    inline TABLESTATUS Get(int nId, unsigned int &nData) { return Get(nId, (int &)nData); }
    inline TABLESTATUS Get(int nId, IDENT &nData) { return Get(nId, (int &)nData); }
    TABLESTATUS Get(int nId, bool &bData);
    TABLESTATUS Seek(int nId, int nSecondId=0);
    void DeleteRecord();

    void Top();
    void Bottom();

    TABLESTATUS Open(const char *strPath);
    void Close();
    void Flush();
    inline int TableType() { return m_nTableType; }

    void Create(const char *strPath, int nTableType);
    void AppendRecord(int nId, int nSecondId=0, int nNewSize=100);
    bool CurrentRecordAtEndOfFile();
    bool Deleted();
    
    void operator=(const CTable &Table);

    bool FindFirst(int nId, int nSecondId=-1);
    bool FindNext(int nId, int nSecondId=-1);

    int NextId();
    int NextSecondId(int nFirstId);
};

#endif






